@@ -85,6 +85,16 @@ static struct keywords boolean_keywords[] =
     { NULL, 0 }
 };
 
+static struct keywords action_keywords[] =
+{
+    { "include", ACTION_TYPE_INCLUDE },
+    { "includeall", ACTION_TYPE_INCLUDE_ALL },
+    { "exclude", ACTION_TYPE_EXCLUDE },
+    { "excludeall", ACTION_TYPE_EXCLUDE_ALL },
+    { "noentry", ACTION_TYPE_NO_ENTRY },
+    { NULL, 0 }
+};
+
 static struct keywords hash_keywords[] =
 {
     { "md5", HASH_MD5 },
@@ -880,7 +890,31 @@ int get_hash_value( char *keyword, unsigned int *value )
     return 0;
 }
 
+/******************************************************************************
+**
+**    Function: get_action_name_from_type
+**
+**    Purpose:  return string pretty name of checksum algorithm, based upon
+**      the constant type given.
+**
+******************************************************************************/
 
+char * get_action_name_from_type( int type )
+{
+    int index;
+    char *name = "";
+
+    for( index = 0; action_keywords[index].word != NULL; index++ )
+    {
+        if ( type == action_keywords[index].type )
+        {
+            name = action_keywords[index].word;
+            return name;
+        }
+    }
+
+    return name;
+}
 
 /******************************************************************************
 **
