@@ -1747,15 +1747,10 @@ osi_bool ctl_save_cmp_filters( CTL_CONTEXT *context )
 
             /* populate the filter structure. */
 
-            osi_strlcpy( f.host, filter->host, sizeof( f.host ) );
-            osi_strlcpy( f.path, filter->path, sizeof( f.path ) );
+            osi_strlcpy( f.exclude, filter->exclude, sizeof( f.exclude ) );
 
-            f.type = filter->type;
-            f.mask = filter->mask;
-            
             /* wrap and pack this filter structure. */
 
-            wrap_cmp_filter( &f );
             size = pack_cmp_filter( &f, buffer, sizeof( buffer ) );
 
             /* send this out in a message, if we are sending the */
@@ -1917,8 +1912,6 @@ osi_list ctl_get_cmp_filters_list( CTL_CONTEXT *context )
                 /* unpack, then unwrap compare filter. */
 
                 unpack_cmp_filter( f, (unsigned char *)filter, (int)size );
-                unwrap_cmp_filter( f );
-
                 list_add( filters, f );
             }
 
