@@ -35,11 +35,11 @@
 #include "md_config.h"
 #include "md_compare.h"
 #include "md_notify.h"
+#include "md_utilities.h"
 
 #include "common.h"
 #include "logging.h"
 
-
 extern OSI_MANAGEMENT_CONFIG *config;
 
 #include <stdlib.h>
@@ -87,13 +87,6 @@ extern OSI_MANAGEMENT_CONFIG *config;
 
 #define NOTIFY_SCHEDULE_FAIL_FORMAT  "The scheduler was unable to initiate the following scan:\r\n\r\nhost: %s\r\nscheduled scan time: %s\r\n\r\nThe scheduler produced the following error message:\r\n\r\n%s"
 
-#define NOTIFY_SESSION_REKEY_FORMAT "The host, \"%s\" seems to have lost its \
-resident session key.\r\nThis key is lost when the agent is restarted \
-or the host is rebooted.\
-\r\n\r\nThis host is configured to generate an alert upon detection of \
-this event.\r\nTo change this, adjust the notification settings for \
-this host.\r\n\r\n"
-
 #define NOTIFY_UPDATE_URL_FORMAT "If these changes are approved, visit the \
 URL below to set the latest scan database to become the trusted database. \
 Alternately, you can login to the management console and set the trusted \
@@ -102,6 +95,13 @@ If these notifications persist, you may need to modify the scan configuration \
 for this host and/or set some comparison filters.\
 \r\n\r\n    <https://%s:%d/?action=v&host=%s&base_db=%s&log=%s>\r\n\r\n"
 
+#define NOTIFY_SESSION_REKEY_FORMAT "The host, \"%s\" seems to have lost its \
+resident session key.\r\nThis key is lost when the agent is restarted \
+or the host is rebooted.\
+\r\n\r\nThis host is configured to generate an alert upon detection of \
+this event.\r\nTo change this, adjust the notification settings for \
+this host.\r\n\r\n"
+
 #define NOTIFY_TEST_MESSAGE "This is a test notification message sent by the \
 Osiris management console.\r\n"
 
@@ -109,8 +109,7 @@ Osiris management console.\r\n"
  * Read a response from the SMTP server
  */
 
-static int
-read_smtp_response( int sock, char *expect )
+int read_smtp_response( int sock, char *expect )
 {
     int i, pos;
     char buf[MAX_SMTP_LINE];
@@ -163,8 +162,8 @@ read_smtp_response( int sock, char *expect )
 /*
  * Write a command string to the SMTP server, checking it's response
  */
-static int
-send_smtp_command(int sock, char *cmd, char *rsp)
+
+int send_smtp_command(int sock, char *cmd, char *rsp)
 {
     /* write a command to the SMTP server */
 
@@ -178,68 +177,7 @@ send_smtp_command(int sock, char *cmd, char *rsp)
     return read_smtp_response(sock, rsp);
 }
 
-/*
- * Open and write the contents of 'logfile' to the given socket
- */
-static int
-send_smtp_file(int sock, char *logfile)
-{
-    FILE *fp = NULL;
-    int len = 0;
-
-    char *line;
-    char buf[OSIRIS_SMTP_CMD_BUFSZ];
-
-    if ((fp = osi_fopen( logfile, "r", LOGFILE_PERMISSIONS)) == NULL)
-    {
-        log_error( LOG_ID_NOTIFY_ERROR, NULL,
-                   "notification failure: error opening log file (%s).",
-                   logfile );
-        return 1;
-    }
-
-    for(;;)
-    {
-        char *temp;
-        line = fgets( buf, sizeof( buf ), fp );
-
-        if( line == NULL)
-        {
-            break;
-        }
-
-	    /* now strip off any newline characters. */
-
-	    if( ( temp = strrchr( buf, '\r' ) ) )
-        {
-            *temp = '\0';
-        }
-
-        if( ( temp = strrchr( buf, '\n' ) ) )
-        {
-            *temp = '\0';
-        }
-
-	    /* now add our own. */
-
-	    osi_strlcat( buf, "\r\n", sizeof( buf ) );
-        len = strlen( buf );
-
-        /* write the buffer to the SMTP server */
-
-        if ((osi_write(sock, buf, len)) == 0)
-        {
-            log_error( LOG_ID_NOTIFY_ERROR, NULL,   
-                       "unable to send data to SMTP server.");
-            break;
-        }
-    }
-
-    fclose(fp);
-    return 0;
-}
-
-static int disconnect_from_mail_server( int sock )
+int disconnect_from_mail_server( int sock )
 {
     int result = 0;
 
@@ -249,7 +187,7 @@ static int disconnect_from_mail_server( int sock )
     return result;
 }
 
-static int connect_to_mail_server( const char *email, OSI_ERROR *error )
+int connect_to_mail_server( const char *email, OSI_ERROR *error )
 {
     int sock = 0;
 
@@ -260,7 +198,6 @@ static int connect_to_mail_server( const char *email, OSI_ERROR *error )
     OSI_ERROR err;
 
     int notify_email = 1;
-    int admin_email  = 1;
 
     /* no error passed in, use a local one. */
 
@@ -274,16 +211,10 @@ static int connect_to_mail_server( const char *email, OSI_ERROR *error )
         notify_email = 0;
     }
 
-    if ( ( config->admin_email == NULL ) ||
-         ( strlen( config->admin_email ) == 0 ) )
-    {
-        admin_email = 0;
-    } 
-
-    if ( notify_email == 0 && admin_email == 0 )
+    if ( notify_email == 0 )
     {
         osi_strlcpy( error->message,
-                     "notification failure: no admin or notify address set!!.",
+                     "notification failure: no notify address set!!.",
                      sizeof( error->message ) );
 
         log_error( LOG_ID_NOTIFY_ERROR, NULL, error->message );
@@ -365,31 +296,8 @@ static int connect_to_mail_server( const char *email, OSI_ERROR *error )
             }
         }
 
-        /* always send to admin. */
-
-        if ( admin_email )
-        {
-            if ( !notify_email )
-            {            
-                osi_snprintf( buf, OSIRIS_SMTP_CMD_BUFSZ,
-                              "MAIL FROM: <%s>\r\n", config->admin_email );
-
-                if ( send_smtp_command( sock, buf, "250" ) )
-                {
-                    break;
-                }
-            }
-
-            osi_snprintf( buf, OSIRIS_SMTP_CMD_BUFSZ, "RCPT TO: <%s>\r\n",
-                          config->admin_email );
-
-            if ( send_smtp_command( sock, buf, "250" ) )
-            {
-                break;
-            }
-        }
-
         /* DATA */
+
         if( send_smtp_command( sock, OSIRIS_SMTP_CMD_DATA, "354" ) )
         {
             break;
@@ -400,305 +308,323 @@ static int connect_to_mail_server( const char *email, OSI_ERROR *error )
     return sock;
 }
 
-void md_notify_session_rekey( const char *host, const char *email )
+static int write_log_file_to_socket( int sock, char *logfile )
 {
-    int sock = 0;
+    FILE *fp = NULL;
+    int len = 0;
 
+    char *line;
     char buf[OSIRIS_SMTP_CMD_BUFSZ];
-    char this_host[MAX_HOSTNAME_LENGTH];
 
-    const char *from_email = email;
-
-    if( host == NULL )
+    if ( ( fp = osi_fopen( logfile, "r", LOGFILE_PERMISSIONS ) ) == NULL )
     {
-        log_error( LOG_ID_NOTIFY_ERROR, "unknown", "null host" );
-        return;
+        log_error( LOG_ID_NOTIFY_ERROR, NULL,
+                   "notification failure: error opening log file (%s).",
+                   logfile );
+        return 1;
     }
 
-    if ( email && strlen( email ) == 0 )
+    for(;;)
     {
-        return;
-    }
+        char *temp;
+        line = fgets( buf, sizeof( buf ), fp );
 
-    if( ( sock = connect_to_mail_server( email, NULL ) ) < 0 )
-    {
-        return;
-    }
-
-    if( config->http_host[0] && strcmp( config->http_host, "" ) != 0 )
-    {
-        osi_snprintf( this_host, sizeof(this_host), "%s", config->http_host );
-    }
-
-    else
-    {
-        gethostname( this_host, sizeof(this_host) );
-    }
-
-    osi_snprintf( buf, OSIRIS_SMTP_CMD_BUFSZ,
-                  /* The following lines are where the default
-                  ** email header is constructed from. This may
-                  ** need to be changed to an actual network account
-                  ** to pass communication validity checks in
-                  ** high-security environments.
-                  */
-                  "To: %s\r\n"
-                  "From: \"Osiris Host Integrity System\" <osirismd@%s>\r\n"
-                  "Date: %s\r\n"
-                  "Subject: [osiris rekey][host: %s] session rekey.\r\n\r\n",
-                  from_email, this_host,
-                  osi_time_to_string(osi_get_time()), host );
-
-    send_smtp_command( sock, buf, NULL );
-
-    /* send a pretty message body. */
-
-    osi_snprintf( buf, sizeof( buf ), NOTIFY_SESSION_REKEY_FORMAT, host );
-
-    osi_write( sock, buf, strlen( buf ) );
-    send_smtp_command( sock, "\r\n.\r\n", NULL );
-    disconnect_from_mail_server( sock );
-    return;
-} 
-
-void md_notify_scan_failed( const char *host,
-                                  const char *email,
-                                  unsigned long scheduled_time,
-                                  const char *message )
-{
-    int sock = 0;
-    char *schedule;
-
-    char buf[OSIRIS_SMTP_CMD_BUFSZ];
-    char this_host[MAX_HOSTNAME_LENGTH];
-
-    const char *from_email = email;
-
-    if( host == NULL )
-    {
-        log_error( LOG_ID_NOTIFY_ERROR, "unknown", "null host" );
-        return;
-    }
-
-    if ( email && strlen( email ) == 0 )
-    {
-        return;
-    }
-
-    /* if we were given a scheduled time, we get it as a string. */
-
-    if( scheduled_time > 0 )
-    {
-        schedule = osi_time_to_string( scheduled_time );
-    }
-
-    else
-    {
-        schedule = "<unknown>";
-    }
-
-    if( ( sock = connect_to_mail_server( email, NULL ) ) < 0 )
-    {
-        return;
-    }
-
-    if( config->http_host[0] && strcmp( config->http_host, "" ) != 0 )
-    {
-        osi_snprintf( this_host, sizeof(this_host), "%s", config->http_host );
-    }             
-                  
-    else          
-    {             
-        gethostname( this_host, sizeof(this_host) );
-    }
-
-    osi_snprintf( buf, OSIRIS_SMTP_CMD_BUFSZ,
-                  /* The following lines are where the default
-                  ** email header is constructed from. This may
-                  ** need to be changed to an actual network account
-                  ** to pass communication validity checks in
-                  ** high-security environments.
-                  */
-                  "To: %s\r\n"
-                  "From: \"Osiris Host Integrity System\" <osirismd@%s>\r\n"
-                  "Date: %s\r\n"
-"Subject: [osiris scheduler][host: %s] failed to start the scheduled scan.\r\n\r\n",
-                  from_email, this_host,
-                  osi_time_to_string(osi_get_time()), host );
-
-    send_smtp_command( sock, buf, NULL );
-
-    /* send a pretty message body. */
-       
-    osi_snprintf( buf, sizeof( buf ), NOTIFY_SCHEDULE_FAIL_FORMAT,
-                  host, schedule, message );
-
-    osi_write( sock, buf, strlen( buf ) );
-    send_smtp_command( sock, "\r\n.\r\n", NULL );
-    disconnect_from_mail_server( sock );    
-
-    return;
-}
-
-void md_notify( const char *email, OSI_DB_CTX *context,osi_bool show_url )
-{
-    int sock = 0;
-    int diff_count = context->diff_count;
-    int smtp_port = config->notify_smtp_port;
-
-    char *smtp_host = config->notify_smtp_host;
-    char *logfile  = context->filepath;
-
-    char buf[OSIRIS_SMTP_CMD_BUFSZ];    /* KISS !malloc */
-    char host[MAX_HOSTNAME_LENGTH];
-
-    const char *from_email = email;
-
-    if (logfile == NULL)
-    {
-        log_error( LOG_ID_NOTIFY_ERROR, context->host,
-                   "notification failure: log file not set." );
-        return;
-    }
-
-    if ( email && strlen( email ) == 0 )
-    {
-        return;
-    }
-
-    if ( ( sock = connect_to_mail_server( email, NULL ) ) < 0  )
-    {
-        return;
-    }
-
-   log_info( LOG_ID_NOTIFY_INFO, context->host,
-            "notifying %s using %s:%d, sending contents of log file: %s, "
-            "containing %d entries.",
-            from_email, smtp_host, smtp_port,logfile, diff_count);
-
-    /* BODY */
-
-    if( config->http_host[0] && strcmp( config->http_host, "" ) != 0 )
-    {
-        osi_snprintf( host, sizeof(host), "%s", config->http_host );
-    }             
-                  
-    else          
-    {             
-        gethostname( host, sizeof(host) );
-    }
-
-    osi_snprintf( buf, OSIRIS_SMTP_CMD_BUFSZ,
-                  /* The following lines are where the default
-                  ** email header is constructed from. This may
-                  ** need to be changed to an actual network account
-                  ** to pass communication validity checks in
-                  ** high-security environments.
-                  */
-                  "To: %s\r\n"
-                  "From: \"Osiris Host Integrity System\" <osirismd@%s>\r\n"
-                  "Date: %s\r\n"
-                  "Subject: [osiris log][host: %s][%d changes]\r\n\r\n",
-                  from_email, host, osi_time_to_string( osi_get_time() ),
-                  context->host, diff_count );
-
-    send_smtp_command( sock, buf, NULL );
-
-    /* before sending the file, we prepend a URL that enables */
-    /* the recipient to approve of these changes by updating  */
-    /* the trusted database via https.                        */
-
-    if( ( config->http_port > 0 ) && ( show_url ) )
-    {
-        char filename[MAX_PATH_LENGTH] = "";
-
-        /* get filename, strip path. */
-
-        osi_get_file_from_path( logfile, filename, sizeof( filename ) );
-
-        if( diff_count > 0 )
+        if( line == NULL)
         {
-            osi_snprintf( buf, sizeof( buf ), NOTIFY_UPDATE_URL_FORMAT,
-                          context->db2_name, host, config->http_port,
-                          context->host, context->db2_name, filename );
+            break;
+        }
 
-            if ((osi_write(sock, buf, strlen(buf))) == 0)
-            {
-                log_error( LOG_ID_NOTIFY_ERROR, NULL,
-                  "notification failure: unable to send data to SMTP server." );
-                return;
-            } 
+        /* now strip off any newline characters. */
+
+        if( ( temp = strrchr( buf, '\r' ) ) )
+        {
+            *temp = '\0';
+        }
+
+        if( ( temp = strrchr( buf, '\n' ) ) )
+        {
+            *temp = '\0';
+        }
+
+        /* now add our own. */
+
+        osi_strlcat( buf, "\r\n", sizeof( buf ) );
+        len = strlen( buf );
+
+        /* write the buffer to the SMTP server */
+
+        if ((osi_write(sock, buf, len)) == 0)
+        {
+            log_error( LOG_ID_NOTIFY_ERROR, NULL,
+                       "unable to send data to SMTP server.");
+            break;
         }
     }
 
-	osi_write( sock, "\r\n\r\n", 4 );
-        
-    /* send the log file contents. */
- 
-    send_smtp_file( sock, logfile );
-    send_smtp_command(sock, "\r\n.\r\n", NULL);
-
-    disconnect_from_mail_server( sock );
-    return;
+    fclose(fp);
+    return 0;
 }
 
-osi_bool md_notify_send_test( const char *email, OSI_ERROR *error )
+
+char * get_notify_type_name( int notify_type )
 {
-    int sock = 0;
+    char * type = NULL;
 
-    char buf[OSIRIS_SMTP_CMD_BUFSZ];
-    char this_host[MAX_HOSTNAME_LENGTH];
-
-    const char *from_email = email;
-
-    if( error == NULL )
+    switch( notify_type )
     {
-        return FALSE;
+        case NOTIFY_TYPE_REPORT:
+            type = "report";
+            break;
+
+        case NOTIFY_TYPE_LOG:
+            type = "log";
+            break;
+
+        case NOTIFY_TYPE_REKEY:
+            type = "rekey";
+            break;
+
+        case NOTIFY_TYPE_TEST:
+            type = "test";
+            break;
+
+        case NOTIFY_TYPE_SCHEDULE_FAIL:
+            type = "schedule_fail";
+            break;
+
+        default:
+            type = "unkonwn";
     }
 
-    if ( email && strlen( email ) == 0 )
+    return type;
+}
+
+
+#ifndef WIN32
+void md_notify_send_notification_to_app( MD_NOTIFY_CTX *ctx )
+{
+    int commpipe[2];
+    char buf[256];
+
+    launch_log_application( commpipe );
+
+    if( ( ctx == NULL ) || ( commpipe[1] <= 0 ) )
     {
-        return FALSE;
+        log_error( LOG_ID_NOTIFY_ERROR, NULL,
+                   "unable to launch notify app comm pipe." );
+        return;
     }
 
-    memset( error, 0, sizeof( OSI_ERROR ) );
+    /* we have an open pipe to the application, send our payloads. */
 
-    if( ( sock = connect_to_mail_server( email, error ) ) < 0 )
+    osi_snprintf( buf, sizeof(buf), "host: %s\ntype: %s\n", ctx->host,
+                  get_notify_type_name( ctx->type ) );
+
+    osi_write( commpipe[1], buf, strlen(buf) );
+
+    if ( ctx->type == NOTIFY_TYPE_REPORT )
     {
-        return FALSE;
+        write_log_file_to_socket( commpipe[1], ctx->filename );
     }
 
-    if( config->http_host[0] && strcmp( config->http_host, "" ) != 0 )
+    else
     {
-        osi_snprintf( this_host, sizeof(this_host), "%s", config->http_host );
-    }             
-                  
-    else          
-    {             
-        gethostname( this_host, sizeof(this_host) );
+        osi_write( commpipe[1], ctx->payload, strlen( ctx->payload ) );
     }
 
-    osi_snprintf( buf, OSIRIS_SMTP_CMD_BUFSZ,
-                  /* The following lines are where the default
-                  ** email header is constructed from. This may
-                  ** need to be changed to an actual network account
-                  ** to pass communication validity checks in
-                  ** high-security environments.
-                  */
+    close_log_application( commpipe );
+}
+#endif
+
+
+void md_notify_send_email( MD_NOTIFY_CTX *ctx )
+{
+    int sock;
+    char buf[2048];
+
+    if ( ctx == NULL )
+    {
+        return;
+    }
+
+    /* first, we create the stupid headers. */
+
+    osi_snprintf( ctx->headers, sizeof(ctx->headers),
                   "To: %s\r\n"
                   "From: \"Osiris Host Integrity System\" <osirismd@%s>\r\n"
-                  "Date: %s\r\n"
+                  "Date: %s\r\n",
+              ctx->email, ctx->hostname, osi_time_to_string( osi_get_time() ) );
+
+    switch( ctx->type )
+    {
+
+        case NOTIFY_TYPE_REPORT:
+        case NOTIFY_TYPE_LOG:
+        {
+            osi_snprintf( buf, sizeof(buf),
+                  "Subject: [osiris log][host: %s][%d changes]\r\n\r\n",
+                  ctx->host, ctx->diff_count );
+
+            osi_strlcat( ctx->headers, buf, sizeof( ctx->headers ) );
+        }
+
+            break;
+
+        case NOTIFY_TYPE_REKEY:
+
+            osi_snprintf( buf, sizeof(buf),
+                  "Subject: [osiris rekey][host: %s] session rekey.\r\n\r\n",
+                  ctx->host );
+
+            osi_strlcat( ctx->headers, buf, sizeof( ctx->headers ) );
+            break;
+
+        case NOTIFY_TYPE_SCHEDULE_FAIL:
+
+            osi_snprintf( buf, sizeof(buf),
+                "Subject: [osiris scheduler][host: %s] failed to scan.\r\n\r\n",
+                ctx->host );
+
+            osi_strlcat( ctx->headers, buf, sizeof( ctx->headers ) );
+            break;
+
+        case NOTIFY_TYPE_TEST:
+
+            osi_strlcat( ctx->headers,
                   "Subject: [osiris test][notification system test]\r\n\r\n",
-                  from_email, this_host, osi_time_to_string( osi_get_time() ) );
+                  sizeof( ctx->headers ) );
+            break;
 
-     send_smtp_command( sock, buf, NULL );
+        default:
+            osi_strlcat( ctx->headers, "\r\n", sizeof( ctx->headers ) );
+            break;
+    }
 
-    /* send a pretty message body. */
+    /* now, we send the mail. */
+
+    if( ( sock = connect_to_mail_server( ctx->email, &(ctx->error) ) ) < 0 )
+    {
+        return;
+    }
+
+    send_smtp_command( sock, ctx->headers, NULL );
+
+    /* send our payload, and the file if we have them. */
+
+    if ( ctx->payload[0] )
+    {
+        osi_write( sock, ctx->payload, strlen( ctx->payload ) );
+    }
+
+    if ( ctx->filename )
+    {
+        write_log_file_to_socket( sock, ctx->filename );
+    }
 
-    osi_strlcpy( buf, NOTIFY_TEST_MESSAGE, sizeof( buf ) );
-    osi_write( sock, buf, strlen( buf ) );
     send_smtp_command( sock, "\r\n.\r\n", NULL );
-
     disconnect_from_mail_server( sock );
-    return TRUE;
 }
+
+void md_create_payload( MD_NOTIFY_CTX *ctx )
+{
+    if ( ctx == NULL )
+    {
+        return;
+    }
+
+    /* setup our hostname. */
+
+    gethostname( ctx->hostname, sizeof(ctx->hostname) );
+
+    switch( ctx->type )
+    {
+        case NOTIFY_TYPE_LOG:
+
+            if( ( config->http_port > 0 ) && ( ctx->show_url ) )
+            {
+                char filename[MAX_PATH_LENGTH] = "";
+
+                /* get filename, strip path. */
+
+                osi_get_file_from_path( ctx->filename, filename,
+                                        sizeof( filename ) );
+
+                osi_snprintf( ctx->payload, sizeof( ctx->payload ),
+                              NOTIFY_UPDATE_URL_FORMAT,
+                              ctx->db_name, ctx->host, config->http_port,
+                              ctx->host, ctx->db_name, filename );
+            }
+
+            break;
+
+        case NOTIFY_TYPE_REKEY:
+
+            osi_snprintf( ctx->payload, sizeof( ctx->payload ),
+                          NOTIFY_SESSION_REKEY_FORMAT, ctx->host );
+
+            break;
+
+        case NOTIFY_TYPE_SCHEDULE_FAIL:
+        {
+            char *now = osi_time_to_string( osi_get_time() );
+
+            osi_snprintf( ctx->payload, sizeof( ctx->payload ),
+                          NOTIFY_SCHEDULE_FAIL_FORMAT, ctx->host,
+                          now, ctx->message );
+        }
+            break;
+
+        case NOTIFY_TYPE_TEST:
+
+            osi_strlcpy( ctx->payload, NOTIFY_TEST_MESSAGE,
+                         sizeof( ctx->payload ) );
+
+            break;
+
+        default:
+            break;
+    }
+}
+
+
+/* right now, the only notification methods are email, or pipe to */
+/* an application, so we check for that information in the ctx.   */
+
+void md_notify( MD_NOTIFY_CTX *ctx )
+{
+    if ( ctx == NULL )
+    {
+        return;
+    }
+        
+    md_create_payload( ctx );
+    
+    /* we have an email target, use it. */
+    
+    if ( ( ctx->email ) && ( strlen( ctx->email ) > 0 ) )
+    {
+        md_notify_send_email( ctx );
+    }
+
+    /* we have an application, pipe our data to it. */
+
+#ifndef WIN32
+    if ( ( config->notify_app ) && ( strlen( config->notify_app ) > 0 ) )
+    {
+        md_notify_send_notification_to_app( ctx );
+    }
+#endif
+}
+
+void md_notify_init( MD_NOTIFY_CTX *ctx, int type )
+{
+    if ( ctx == NULL )
+    {
+        return;
+    }
+
+    memset( ctx, 0, sizeof(MD_NOTIFY_CTX) );
+    ctx->type = type;
+}
+
+
