@@ -362,6 +362,10 @@ THREAD_FUNCTION_TYPE handle_scan_stream( void *scan_context )
                 unwrap_error( error );
                 result = osi_scan_db_store_error( &db, error );
 
+                /* log this error to system log as well. */
+
+                log_error( LOG_ID_SCAN_ERROR, host->name, error->message ); 
+
                 if( result != OSI_DB_OK )
                 {
                     log_error( LOG_ID_DB_STORE_ERROR, host->name,
