@@ -110,12 +110,14 @@ THREAD_FUNCTION_TYPE run( void *unused )
     verify_host_directories();
 
     initialize_ssl();
-    start_scheduler();
+    initialize_signals();
 
     fork_daemon_process();
+
+    start_log_application();
+    start_scheduler();
     start_servers();
 
-    initialize_signals();
     md_auth_load_database();
 
     process();
@@ -819,6 +821,10 @@ void halt( int code )
 
     md_auth_unload_database();
 
+#ifndef WIN32
+    close_log_application( log_pipe );
+#endif
+
 #ifdef WIN32
     WSACleanup();
     DeregisterEventSource( event_source );
@@ -1080,6 +1086,13 @@ int ssl_verify_callback( int ok, X509_STORE_CTX *store )
     return ok;
 }
 
+void start_log_application()
+{
+#ifndef WIN32
+    log_pid = launch_log_application( log_pipe );
+#endif
+}
+
 /******************************************************************************
 **
 **    Function: start_servers
@@ -1485,10 +1498,18 @@ void fork_daemon_process()
     if( pid == 0 )
     {
         log_info( LOG_ID_DAEMON_INFO, NULL, "daemon process spawned." );
+       
+#ifdef HAVE_SETSID 
+        if ( setsid() < 0 )
+        {
+            log_error( LOG_ID_DAEMON_ERROR, NULL, "unable to spawn session." );
+        }
+#endif
     }
 #endif
 }
 
+
 /******************************************************************************
 **
 **    Function: initialize_pipes
@@ -1532,7 +1553,6 @@ void initialize_pipes()
         log_error( LOG_ID_DAEMON_ERROR, NULL, "creating restart pipe!" );
         halt( EXIT_CODE_ERROR );
     }
-
 #endif
 }
 
@@ -1646,13 +1666,7 @@ void handle_sigchld( int signal )
 void handle_sigpipe( int signal )
 {
 #ifndef WIN32
-    if( getpid() == scheduler_pid )
-    {
-        exit( EXIT_CODE_ERROR );
-    }
-
     received_sigpipe = 1;
-
 #endif
 }
 
@@ -1720,26 +1734,48 @@ void check_for_signals()
     if( received_sighup )
     {
         received_sighup = 0;
-        handle_restart();
+
+        if ( getpid() != scheduler_pid )
+        {
+            handle_restart();
+        }
     }
 
     if( received_sigchld )
     {
         int status = 0;
-        pid_t pid  = 0;
+        pid_t child_pid  = 0;
 
         received_sigchld = 0;
 
         /* wait for child to finish. */
 
+        child_pid = wait(&status);
+
+/*
         while( ( pid = waitpid( -1, &status, WNOHANG ) ) > 0 ||
                ( ( pid < 0 ) && ( errno == EINTR ) ) )
         ;
+*/
 
-        if( pid == scheduler_pid )
+        if( child_pid == scheduler_pid )
         {
             log_error( LOG_ID_DAEMON_INFO, NULL,
                        "SIGCHLD, scheduler has terminated!" );
+
+            halt(EXIT_CODE_ERROR);
+        }
+
+        else if ( child_pid == log_pid )
+        {
+            log_error( LOG_ID_DAEMON_INFO, NULL,
+                       "SIGCHLD, log application was terminated!" );
+        }
+
+        else if ( getpid() == scheduler_pid )
+        {
+            log_info( LOG_ID_SCHEDULER_INFO, NULL,
+                      "SIGCHLD, scheduler child process has terminated." );
         }
 
         else
